@@ -15,6 +15,7 @@
 #include "common/Thread.h"
 #include "common/admin_socket.h"
 #include "common/config.h"
+#include "common/cmdparse.h"
 #include "common/dout.h"
 #include "common/errno.h"
 #include "common/perf_counters.h"
@@ -44,6 +45,7 @@
 #undef dout_prefix
 #define dout_prefix *_dout << "asok(" << (void*)m_cct << ") "
 
+
 using std::ostringstream;
 
 /*
@@ -251,7 +253,6 @@ void* AdminSocket::entry()
 
 bool AdminSocket::do_accept()
 {
-  int ret;
   struct sockaddr_un address;
   socklen_t address_length = sizeof(address);
   ldout(m_cct, 30) << "AdminSocket: calling accept" << dendl;
@@ -269,7 +270,7 @@ bool AdminSocket::do_accept()
   int pos = 0;
   string c;
   while (1) {
-    ret = safe_read(connection_fd, &cmd[pos], 1);
+    int ret = safe_read(connection_fd, &cmd[pos], 1);
     if (ret <= 0) {
       lderr(m_cct) << "AdminSocket: error reading request code: "
 		   << cpp_strerror(ret) << dendl;
@@ -356,8 +357,7 @@ bool AdminSocket::do_accept()
       lderr(m_cct) << "AdminSocket: error writing response length "
 		   << cpp_strerror(ret) << dendl;
     } else {
-      ret = out.write_fd(connection_fd);
-      if (ret >= 0)
+      if (out.write_fd(connection_fd) >= 0)
 	rval = true;
     }
   }
@@ -367,7 +367,7 @@ bool AdminSocket::do_accept()
   return rval;
 }
 
-int AdminSocket::register_command(std::string command, AdminSocketHook *hook, std::string help)
+int AdminSocket::register_command(std::string command, std::string cmddesc, AdminSocketHook *hook, std::string help)
 {
   int ret;
   m_lock.Lock();
@@ -377,6 +377,7 @@ int AdminSocket::register_command(std::string command, AdminSocketHook *hook, st
   } else {
     ldout(m_cct, 5) << "register_command " << command << " hook " << hook << dendl;
     m_hooks[command] = hook;
+    m_descs[command] = cmddesc;
     if (help.length())
       m_help[command] = help;
     ret = 0;
@@ -392,6 +393,7 @@ int AdminSocket::unregister_command(std::string command)
   if (m_hooks.count(command)) {
     ldout(m_cct, 5) << "unregister_command " << command << dendl;
     m_hooks.erase(command);
+    m_descs.erase(command);
     m_help.erase(command);
     ret = 0;
   } else {
@@ -443,6 +445,33 @@ public:
   }
 };
 
+class GetdescsHook : public AdminSocketHook {
+  AdminSocket *m_as;
+public:
+  GetdescsHook(AdminSocket *as) : m_as(as) {}
+  bool call(string command, string args, bufferlist& out) {
+    int cmdnum = 0;
+    JSONFormatter jf(false);
+    jf.open_object_section("command_descriptions");
+    for (map<string,string>::iterator p = m_as->m_descs.begin();
+	 p != m_as->m_descs.end();
+	 ++p) {
+      ostringstream secname;
+      secname << "cmd" << setfill('0') << std::setw(3) << cmdnum;
+      dump_cmd_and_help_to_json(&jf,
+				secname.str().c_str(),
+				p->second.c_str(),
+				m_as->m_help[p->first]);
+      cmdnum++;
+    }
+    jf.close_section(); // command_descriptions
+    ostringstream ss;
+    jf.flush(ss);
+    out.append(ss.str());
+    return true;
+  }
+};
+
 bool AdminSocket::init(const std::string &path)
 {
   ldout(m_cct, 5) << "init " << path << dendl;
@@ -471,11 +500,14 @@ bool AdminSocket::init(const std::string &path)
   m_path = path;
 
   m_version_hook = new VersionHook;
-  register_command("0", m_version_hook, "");
-  register_command("version", m_version_hook, "get ceph version");
-  register_command("git_version", m_version_hook, "get git sha1");
+  register_command("0", "0", m_version_hook, "");
+  register_command("version", "version", m_version_hook, "get ceph version");
+  register_command("git_version", "git_version", m_version_hook, "get git sha1");
   m_help_hook = new HelpHook(this);
-  register_command("help", m_help_hook, "list available commands");
+  register_command("help", "help", m_help_hook, "list available commands");
+  m_getdescs_hook = new GetdescsHook(this);
+  register_command("get_command_descriptions", "get_command_descriptions",
+		   m_getdescs_hook, "list available commands");
 
   create();
   add_cleanup_file(m_path.c_str());
@@ -508,6 +540,8 @@ void AdminSocket::shutdown()
   delete m_version_hook;
   unregister_command("help");
   delete m_help_hook;
+  unregister_command("get_command_descriptions");
+  delete m_getdescs_hook;
 
   remove_cleanup_file(m_path.c_str());
   m_path.clear();
