@@ -50,14 +50,15 @@ static const struct sockaddr *find_ip_in_subnet_list(CephContext *cct,
 
 // observe this change
 struct Observer : public md_config_obs_t {
-  const char *keys[2];
-  Observer(const char *c) {
-    keys[0] = c;
-    keys[1] = NULL;
-  }
+  const char *conf_var;
+  Observer(const char *c) : conf_var(c) {}
 
   const char** get_tracked_conf_keys() const {
-    return (const char **)keys;
+    static const char *foo[] = {
+      conf_var,
+      NULL
+    };
+    return foo;
   }
   void handle_conf_change(const struct md_config_t *conf,
 			  const std::set <std::string> &changed) {
@@ -102,7 +103,7 @@ static void fill_in_one_address(CephContext *cct,
   cct->_conf->remove_observer(&obs);
 }
 
-void pick_addresses(CephContext *cct)
+void pick_addresses(CephContext *cct, int needs)
 {
   struct ifaddrs *ifa;
   int r = getifaddrs(&ifa);
@@ -112,11 +113,15 @@ void pick_addresses(CephContext *cct)
     exit(1);
   }
 
-  if (cct->_conf->public_addr.is_blank_ip() && !cct->_conf->public_network.empty()) {
+  if ((needs & CEPH_PICK_ADDRESS_PUBLIC)
+      && cct->_conf->public_addr.is_blank_ip()
+      && !cct->_conf->public_network.empty()) {
     fill_in_one_address(cct, ifa, cct->_conf->public_network, "public_addr");
   }
 
-  if (cct->_conf->cluster_addr.is_blank_ip() && !cct->_conf->cluster_network.empty()) {
+  if ((needs & CEPH_PICK_ADDRESS_CLUSTER)
+      && cct->_conf->cluster_addr.is_blank_ip()
+      && !cct->_conf->cluster_network.empty()) {
     fill_in_one_address(cct, ifa, cct->_conf->cluster_network, "cluster_addr");
   }
 
