@@ -11,6 +11,7 @@
 #include <string>
 #include <stdexcept>
 #include <map>
+#include <cctype>
 
 #include <typeinfo>
 
@@ -99,12 +100,9 @@ int CrushCompiler::decompile_bucket_impl(int i, ostream &out)
     print_item_name(out, item, crush);
     out << " weight ";
     print_fixedpoint(out, w);
-    if (dopos) {
-      if (alg == CRUSH_BUCKET_TREE)
-	out << " pos " << j;
-      else
-	out << " pos " << j;
-    }
+    if (dopos) 
+      out << " pos " << j;
+    
     out << "\n";
   }
   out << "}\n";
@@ -179,9 +177,19 @@ int CrushCompiler::decompile_bucket(int cur,
 
 int CrushCompiler::decompile(ostream &out)
 {
-  out << "# begin crush map\n\n";
+  out << "# begin crush map\n";
 
-  out << "# devices\n";
+  // only dump tunables if they differ from the defaults
+  if (crush.get_choose_local_tries() != 2)
+    out << "tunable choose_local_tries " << crush.get_choose_local_tries() << "\n";
+  if (crush.get_choose_local_fallback_tries() != 5)
+    out << "tunable choose_local_fallback_tries " << crush.get_choose_local_fallback_tries() << "\n";
+  if (crush.get_choose_total_tries() != 19)
+    out << "tunable choose_total_tries " << crush.get_choose_total_tries() << "\n";
+  if (crush.get_chooseleaf_descend_once() != 0)
+    out << "tunable chooseleaf_descend_once " << crush.get_chooseleaf_descend_once() << "\n";
+
+  out << "\n# devices\n";
   for (int i=0; i<crush.get_max_devices(); i++) {
     out << "device " << i << " ";
     print_item_name(out, i, crush);
@@ -322,6 +330,33 @@ int CrushCompiler::parse_device(iter_t const& i)
   return 0;
 }
 
+int CrushCompiler::parse_tunable(iter_t const& i)
+{
+  string name = string_node(i->children[1]);
+  int val = int_node(i->children[2]);
+
+  if (name == "choose_local_tries")
+    crush.set_choose_local_tries(val);
+  else if (name == "choose_local_fallback_tries")
+    crush.set_choose_local_fallback_tries(val);
+  else if (name == "choose_total_tries")
+    crush.set_choose_total_tries(val);
+  else if (name == "chooseleaf_descend_once")
+    crush.set_chooseleaf_descend_once(val);
+  else {
+    err << "tunable " << name << " not recognized" << std::endl;
+    return -1;
+  }
+
+  if (!unsafe_tunables) {
+    err << "tunables are NOT FULLY IMPLEMENTED; enable with --enable-unsafe-tunables to enable this feature" << std::endl;
+    return -1;
+  }
+
+  if (verbose) err << "tunable " << name << " " << val << std::endl;
+  return 0;
+}
+
 int CrushCompiler::parse_bucket_type(iter_t const& i)
 {
   int id = int_node(i->children[1]);
@@ -406,9 +441,9 @@ int CrushCompiler::parse_bucket(iter_t const& i)
   vector<int> weights(size);
 
   int curpos = 0;
-  float bucketweight = 0;
+  unsigned bucketweight = 0;
   bool have_uniform_weight = false;
-  float uniform_weight = 0;
+  unsigned uniform_weight = 0;
   for (unsigned p=3; p<i->children.size()-1; p++) {
     iter_t sub = i->children.begin() + p;
     string tag = string_node(sub->children[0]);
@@ -421,19 +456,30 @@ int CrushCompiler::parse_bucket(iter_t const& i)
       }
       int itemid = item_id[iname];
 
-      float weight = 1.0;
+      unsigned weight = 0x10000;
       if (item_weight.count(itemid))
 	weight = item_weight[itemid];
 
       int pos = -1;
       for (unsigned q = 2; q < sub->children.size(); q++) {
 	string tag = string_node(sub->children[q++]);
-	if (tag == "weight")
-	  weight = float_node(sub->children[q]);
+	if (tag == "weight") {
+	  weight = float_node(sub->children[q]) * (float)0x10000;
+	  if (weight > CRUSH_MAX_DEVICE_WEIGHT && itemid >= 0) {
+	    err << "device weight limited to " << CRUSH_MAX_DEVICE_WEIGHT / 0x10000 << std::endl;
+	    return -ERANGE;
+	  }
+	  else if (weight > CRUSH_MAX_BUCKET_WEIGHT && itemid < 0) {
+	    err << "bucket weight limited to " << CRUSH_MAX_BUCKET_WEIGHT / 0x10000
+	        << " to prevent overflow" << std::endl;
+	    return -ERANGE;
+	  }
+	}
 	else if (tag == "pos") 
 	  pos = int_node(sub->children[q]);
 	else
 	  assert(0);
+
       }
       if (alg == CRUSH_BUCKET_UNIFORM) {
 	if (!have_uniform_weight) {
@@ -442,7 +488,7 @@ int CrushCompiler::parse_bucket(iter_t const& i)
 	} else {
 	  if (uniform_weight != weight) {
 	    err << "item '" << iname << "' in uniform bucket '" << name << "' has weight " << weight
-		<< " but previous item(s) have weight " << uniform_weight
+		<< " but previous item(s) have weight " << (float)uniform_weight/(float)0x10000
 		<< "; uniform bucket items must all have identical weights." << std::endl;
 	    return -1;
 	  }
@@ -459,7 +505,13 @@ int CrushCompiler::parse_bucket(iter_t const& i)
       }
       //err << " item " << iname << " (" << itemid << ") pos " << pos << " weight " << weight << std::endl;
       items[pos] = itemid;
-      weights[pos] = (unsigned)(weight * 0x10000);
+      weights[pos] = weight;
+
+      if (crush_addition_is_unsafe(bucketweight, weight)) {
+        err << "oh no! our bucket weights are overflowing all over the place, better lower the item weights" << std::endl;
+        return -ERANGE;
+      }
+
       bucketweight += weight;
     }
   }
@@ -469,7 +521,8 @@ int CrushCompiler::parse_bucket(iter_t const& i)
     //err << "assigned id " << id << std::endl;
   }
 
-  if (verbose) err << "bucket " << name << " (" << id << ") " << size << " items and weight " << bucketweight << std::endl;
+  if (verbose) err << "bucket " << name << " (" << id << ") " << size << " items and weight "
+		   << (float)bucketweight / (float)0x10000 << std::endl;
   id_item[id] = name;
   item_id[name] = id;
   item_weight[id] = bucketweight;
@@ -595,6 +648,9 @@ int CrushCompiler::parse_crush(iter_t const& i)
   int r = 0;
   for (iter_t p = i->children.begin(); p != i->children.end(); p++) {
     switch (p->value.id().to_long()) {
+    case crush_grammar::_tunable:
+      r = parse_tunable(p);
+      break;
     case crush_grammar::_device: 
       r = parse_device(p);
       break;
@@ -621,13 +677,14 @@ int CrushCompiler::parse_crush(iter_t const& i)
   return 0;
 } 
 
+// squash runs of whitespace to one space, excepting newlines
 string CrushCompiler::consolidate_whitespace(string in)
 {
   string out;
 
   bool white = false;
   for (unsigned p=0; p<in.length(); p++) {
-    if (in[p] == ' ' || in[p] == '\t') {
+    if (isspace(in[p]) && in[p] != '\n') {
       if (white)
 	continue;
       white = true;
@@ -711,7 +768,7 @@ int CrushCompiler::compile(istream& in, const char *infn)
     assert(!line_pos.empty());
     map<int,int>::iterator p = line_pos.upper_bound(cpos);
     if (p != line_pos.begin())
-      p--;
+      --p;
     int line = p->second;
     int pos = cpos - p->first;
     err << infn << ":" << line //<< ":" << (pos+1)
