@@ -6,6 +6,8 @@
 #include "mdstypes.h"
 #include "mds/flock.h"
 
+#define dout_subsys ceph_subsys_mds
+
 bool ceph_lock_state_t::is_waiting(ceph_filelock &fl)
 {
   multimap<uint64_t, ceph_filelock>::iterator p = waiting_locks.find(fl.start);
@@ -122,14 +124,13 @@ void ceph_lock_state_t::remove_lock(ceph_filelock removal_lock,
                  list<ceph_filelock>& activated_locks)
 {
   list<multimap<uint64_t, ceph_filelock>::iterator> overlapping_locks,
-    self_overlapping_locks, crossed_waiting_locks;
+    self_overlapping_locks;
   if (get_overlapping_locks(removal_lock, overlapping_locks)) {
     dout(15) << "splitting by owner" << dendl;
     split_by_owner(removal_lock, overlapping_locks, self_overlapping_locks);
   } else dout(15) << "attempt to remove lock at " << removal_lock.start
                  << " but no locks there!" << dendl;
   bool remove_to_end = (0 == removal_lock.length);
-  bool old_lock_to_end;
   uint64_t removal_start = removal_lock.start;
   uint64_t removal_end = removal_start + removal_lock.length - 1;
   uint64_t old_lock_end;
@@ -144,7 +145,7 @@ void ceph_lock_state_t::remove_lock(ceph_filelock removal_lock,
        ++iter) {
     dout(15) << "self overlapping lock " << (*iter)->second << dendl;
     old_lock = &(*iter)->second;
-    old_lock_to_end = (0 == old_lock->length);
+    bool old_lock_to_end = (0 == old_lock->length);
     old_lock_end = old_lock->start + old_lock->length - 1;
     old_lock_client = old_lock->client;
     if (remove_to_end) {
@@ -211,7 +212,6 @@ void ceph_lock_state_t::adjust_locks(list<multimap<uint64_t, ceph_filelock>::ite
 {
   dout(15) << "adjust_locks" << dendl;
   bool new_lock_to_end = (0 == new_lock.length);
-  bool old_lock_to_end;
   uint64_t new_lock_start = new_lock.start;
   uint64_t new_lock_end = new_lock.start + new_lock.length - 1;
   uint64_t old_lock_start, old_lock_end;
@@ -223,7 +223,7 @@ void ceph_lock_state_t::adjust_locks(list<multimap<uint64_t, ceph_filelock>::ite
        ++iter) {
     old_lock = &(*iter)->second;
     dout(15) << "adjusting lock: " << *old_lock << dendl;
-    old_lock_to_end = (0 == old_lock->length);
+    bool old_lock_to_end = (0 == old_lock->length);
     old_lock_start = old_lock->start;
     old_lock_end = old_lock->start + old_lock->length - 1;
     new_lock_start = new_lock.start;
