@@ -35,7 +35,6 @@ class KeyRing;
 #define MIN_GLOBAL_ID 0x1000
 
 class AuthMonitor : public PaxosService {
-  void auth_usage(stringstream& ss);
   enum IncType {
     GLOBAL_ID,
     AUTH_DATA,
@@ -130,7 +129,7 @@ private:
   void on_active();
   bool should_propose(double& delay);
   void create_initial();
-  void update_from_paxos();
+  void update_from_paxos(bool *need_bootstrap);
   void create_pending();  // prepare a new pending
   bool prepare_global_id(MMonGlobalID *m);
   void increase_max_global_id();
