@@ -63,15 +63,19 @@ class MDSMonitor : public PaxosService {
     }
   };
 
-
   void create_new_fs(MDSMap &m, int metadata_pool, int data_pool);
 
 
   // service methods
   void create_initial();
-  void update_from_paxos();
+  void update_from_paxos(bool *need_bootstrap);
   void create_pending(); 
-  void encode_pending(bufferlist &bl);
+  void encode_pending(MonitorDBStore::Transaction *t);
+  // we don't require full versions; don't encode any.
+  virtual void encode_full(MonitorDBStore::Transaction *t) { }
+
+  bool service_should_trim() { return false; }
+  void encode_trim(MonitorDBStore::Transaction *t) { }
 
   void update_logger();
 
@@ -95,8 +99,6 @@ class MDSMonitor : public PaxosService {
   int fail_mds(std::ostream &ss, const std::string &arg);
   void fail_mds_gid(uint64_t gid);
 
-  int cluster_fail(std::ostream &ss);
-
   bool preprocess_command(MMonCommand *m);
   bool prepare_command(MMonCommand *m);
 
@@ -110,13 +112,12 @@ class MDSMonitor : public PaxosService {
   bool try_standby_replay(MDSMap::mds_info_t& finfo, MDSMap::mds_info_t& ainfo);
 
 public:
-  MDSMonitor(Monitor *mn, Paxos *p)
-    : PaxosService(mn, p)
+  MDSMonitor(Monitor *mn, Paxos *p, string service_name)
+    : PaxosService(mn, p, service_name)
   {
   }
 
   void tick();     // check state, take actions
-  void do_stop();
 
   void dump_info(Formatter *f);
 
