@@ -120,7 +120,6 @@ void PGMonitor::tick()
 {
   if (!is_active()) return;
 
-  update_from_paxos();
   handle_osd_timeouts();
 
   if (mon->is_leader()) {
@@ -150,7 +149,7 @@ void PGMonitor::create_initial()
   dout(10) << "create_initial -- creating initial map" << dendl;
 }
 
-void PGMonitor::update_from_paxos()
+void PGMonitor::update_from_paxos(bool *need_bootstrap)
 {
   version_t version = get_version();
   if (version == pg_map.version)
@@ -1604,6 +1603,6 @@ void PGMonitor::check_sub(Subscription *sub)
 {
   if (sub->type == "osd_pg_creates") {
     send_pg_creates(sub->session->inst.name.num(),
-		    sub->session->con);
+		    sub->session->con.get());
   }
 }
