@@ -52,7 +52,7 @@ private:
 
   void create_initial();
   void update_from_paxos(bool *need_bootstrap);
-  void post_paxos_update();
+  void init();
   void handle_osd_timeouts();
   void create_pending();  // prepare a new pending
   // propose pending update to peers
@@ -128,9 +128,9 @@ private:
    *
    * @return 0 on success, negative error code on failure
    */
-  int dump_stuck_pg_stats(ostream& ss,
-			  bufferlist& rdata,
-			  vector<const char*>& args) const;
+  int dump_stuck_pg_stats(stringstream &ds, Formatter *f,
+			  int threshold,
+			  vector<string>& args) const;
 
   void dump_object_stat_sum(TextTable &tbl, Formatter *f,
                             object_stat_sum_t &sum, bool verbose);
