@@ -21,6 +21,7 @@
 
 #include "common/config.h"
 #include "include/assert.h"
+#include "include/stringify.h"
 #include "common/Formatter.h"
 
 #define dout_subsys ceph_subsys_paxos
@@ -37,13 +38,6 @@ static ostream& _prefix(std::ostream *_dout, Monitor *mon, const string& name,
 		<< ") ";
 }
 
-void Paxos::prepare_bootstrap()
-{
-  dout(0) << __func__ << dendl;
-
-  going_to_bootstrap = true;
-}
-
 MonitorDBStore *Paxos::get_store()
 {
   return mon->store;
@@ -437,14 +431,15 @@ void Paxos::handle_last(MMonPaxos *last)
       if (uncommitted_v == last_committed+1 &&
 	  uncommitted_value.length()) {
 	dout(10) << "that's everyone.  begin on old learned value" << dendl;
-	state = STATE_PREPARING | STATE_LOCKED;
+	state = STATE_UPDATING_PREVIOUS;
 	begin(uncommitted_value);
       } else {
 	// active!
-	state = STATE_ACTIVE;
 	dout(10) << "that's everyone.  active!" << dendl;
 	extend_lease();
 
+        finish_proposal();
+
 	finish_contexts(g_ceph_context, waiting_for_active);
 	finish_contexts(g_ceph_context, waiting_for_readable);
 	finish_contexts(g_ceph_context, waiting_for_writeable);
@@ -475,9 +470,7 @@ void Paxos::begin(bufferlist& v)
 	   << dendl;
 
   assert(mon->is_leader());
-  assert(is_preparing());
-  state &= ~STATE_PREPARING;
-  state |= STATE_UPDATING;
+  assert(is_updating() || is_updating_previous());
 
   // we must already have a majority for this to work.
   assert(mon->get_quorum().size() == 1 ||
@@ -512,13 +505,11 @@ void Paxos::begin(bufferlist& v)
   if (mon->get_quorum().size() == 1) {
     // we're alone, take it easy
     commit();
-    state = STATE_ACTIVE;
     finish_proposal();
     finish_contexts(g_ceph_context, waiting_for_active);
     finish_contexts(g_ceph_context, waiting_for_commit);
     finish_contexts(g_ceph_context, waiting_for_readable);
     finish_contexts(g_ceph_context, waiting_for_writeable);
-
     return;
   }
 
@@ -608,7 +599,7 @@ void Paxos::handle_accept(MMonPaxos *accept)
   assert(accept->last_committed == last_committed ||   // not committed
 	 accept->last_committed == last_committed-1);  // committed
 
-  assert(is_updating());
+  assert(is_updating() || is_updating_previous());
   assert(accepted.count(from) == 0);
   accepted.insert(from);
   dout(10) << " now " << accepted << " have accepted" << dendl;
@@ -629,9 +620,8 @@ void Paxos::handle_accept(MMonPaxos *accept)
     accept_timeout_event = 0;
 
     // yay!
-    state = STATE_ACTIVE;
     extend_lease();
-  
+
     finish_proposal();
 
     // wake people up
@@ -648,7 +638,7 @@ void Paxos::accept_timeout()
   dout(5) << "accept timeout, calling fresh election" << dendl;
   accept_timeout_event = 0;
   assert(mon->is_leader());
-  assert(is_updating());
+  assert(is_updating() || is_updating_previous());
   mon->bootstrap();
 }
 
@@ -717,16 +707,18 @@ void Paxos::handle_commit(MMonPaxos *commit)
   }
 
   store_state(commit);
-  
+
   commit->put();
 
+  mon->refresh_from_paxos(NULL);
+
   finish_contexts(g_ceph_context, waiting_for_commit);
 }
 
 void Paxos::extend_lease()
 {
   assert(mon->is_leader());
-  assert(is_active());
+  //assert(is_active());
 
   lease_expire = ceph_clock_now(g_ceph_context);
   lease_expire += g_conf->mon_lease;
@@ -784,40 +776,34 @@ void Paxos::warn_on_future_time(utime_t t, entity_name_t from)
 
 }
 
-void Paxos::finish_queued_proposal()
-{
-  assert(mon->is_leader());
-  assert(!proposals.empty());
-
-  dout(10) << __func__ << " finishing proposal" << dendl;
-  C_Proposal *proposal = static_cast<C_Proposal*>(proposals.front());
-  dout(10) << __func__ << " finish it (proposal = "
-	   << proposal << ")" << dendl;;
-
-  assert(proposal != NULL);
-
-  if (!proposal->proposed) {
-    dout(10) << __func__ << " we must have received a stay message and we're "
-             << "trying to finish before time. "
-	     << "Instead, propose it (if we are active)!" << dendl;
-  } else {
-    dout(10) << __func__ << " proposal took "
-	     << (ceph_clock_now(NULL) - proposal->proposal_time)
-	     << " to finish" << dendl;
-
-    proposals.pop_front();
-    proposal->complete(0);
-  }
-}
-
 void Paxos::finish_proposal()
 {
-  /* There is a lot of debug still going around. We will get rid of it later
-   * on, as soon as everything "just works (tm)"
-   */
   assert(mon->is_leader());
-  if (!proposals.empty())
-    finish_queued_proposal();
+
+  // make sure we have the latest state loaded up
+  bool need_bootstrap = false;
+  mon->refresh_from_paxos(&need_bootstrap);
+
+  // ok, now go active!
+  state = STATE_ACTIVE;
+
+  // finish off the last proposal
+  if (!proposals.empty()) {
+    assert(mon->is_leader());
+
+    C_Proposal *proposal = static_cast<C_Proposal*>(proposals.front());
+    if (!proposal->proposed) {
+      dout(10) << __func__ << " proposal " << proposal << ": we must have received a stay message and we're "
+	       << "trying to finish before time. "
+	       << "Instead, propose it (if we are active)!" << dendl;
+    } else {
+      dout(10) << __func__ << " proposal " << proposal << " took "
+	       << (ceph_clock_now(NULL) - proposal->proposal_time)
+	       << " to finish" << dendl;
+      proposals.pop_front();
+      proposal->complete(0);
+    }
+  }
 
   dout(10) << __func__ << " state " << state
 	   << " proposals left " << proposals.size() << dendl;
@@ -834,8 +820,8 @@ void Paxos::finish_proposal()
   first_committed = get_store()->get(get_name(), "first_committed");
   last_committed = get_store()->get(get_name(), "last_committed");
 
-  if (proposals.empty() && going_to_bootstrap) {
-    dout(0) << __func__ << " no more proposals; bootstraping." << dendl;
+  if (need_bootstrap) {
+    dout(10) << " doing requested bootstrap" << dendl;
     mon->bootstrap();
     return;
   }
@@ -888,10 +874,7 @@ void Paxos::handle_lease(MMonPaxos *lease)
   mon->messenger->send_message(ack, lease->get_source_inst());
 
   // (re)set timeout event.
-  if (lease_timeout_event) 
-    mon->timer.cancel_event(lease_timeout_event);
-  lease_timeout_event = new C_LeaseTimeout(this);
-  mon->timer.add_event_after(g_conf->mon_lease_ack_timeout, lease_timeout_event);
+  reset_lease_timeout();
 
   // kick waiters
   finish_contexts(g_ceph_context, waiting_for_active);
@@ -946,6 +929,15 @@ void Paxos::lease_ack_timeout()
   mon->bootstrap();
 }
 
+void Paxos::reset_lease_timeout()
+{
+  dout(20) << "reset_lease_timeout - setting timeout event" << dendl;
+  if (lease_timeout_event)
+    mon->timer.cancel_event(lease_timeout_event);
+  lease_timeout_event = new C_LeaseTimeout(this);
+  mon->timer.add_event_after(g_conf->mon_lease_ack_timeout, lease_timeout_event);
+}
+
 void Paxos::lease_timeout()
 {
   dout(5) << "lease_timeout -- calling new election" << dendl;
@@ -970,14 +962,13 @@ void Paxos::trim_to(MonitorDBStore::Transaction *t,
   dout(10) << __func__ << " from " << from << " to " << to << dendl;
   assert(from < to);
 
-  while (from < to) {
-    dout(10) << "trim " << from << dendl;
-    t->erase(get_name(), from);
-    from++;
+  for (version_t v = from; v < to; ++v) {
+    dout(10) << "trim " << v << dendl;
+    t->erase(get_name(), v);
   }
   if (g_conf->mon_compact_on_trim) {
-    dout(10) << " compacting prefix" << dendl;
-    t->compact_prefix(get_name());
+    dout(10) << " compacting trimmed range" << dendl;
+    t->compact_range(get_name(), stringify(from - 1), stringify(to));
   }
 }
 
@@ -1085,16 +1076,15 @@ void Paxos::shutdown() {
   finish_contexts(g_ceph_context, waiting_for_commit, -ECANCELED);
   finish_contexts(g_ceph_context, waiting_for_readable, -ECANCELED);
   finish_contexts(g_ceph_context, waiting_for_active, -ECANCELED);
+  finish_contexts(g_ceph_context, proposals, -ECANCELED);
 }
 
 void Paxos::leader_init()
 {
   cancel_events();
   new_value.clear();
-  if (!proposals.empty())
-    proposals.clear();
 
-  going_to_bootstrap = false;
+  finish_contexts(g_ceph_context, proposals, -EAGAIN);
 
   if (mon->get_quorum().size() == 1) {
     state = STATE_ACTIVE;
@@ -1116,9 +1106,13 @@ void Paxos::peon_init()
   lease_expire = utime_t();
   dout(10) << "peon_init -- i am a peon" << dendl;
 
+  // start a timer, in case the leader never manages to issue a lease
+  reset_lease_timeout();
+
   // no chance to write now!
   finish_contexts(g_ceph_context, waiting_for_writeable, -EAGAIN);
   finish_contexts(g_ceph_context, waiting_for_commit, -EAGAIN);
+  finish_contexts(g_ceph_context, proposals, -EAGAIN);
 }
 
 void Paxos::restart()
@@ -1126,13 +1120,10 @@ void Paxos::restart()
   dout(10) << "restart -- canceling timeouts" << dendl;
   cancel_events();
   new_value.clear();
-  dout(10) << __func__ << " -- clearing queued proposals" << dendl;
-  if (!proposals.empty())
-    proposals.clear();
 
   state = STATE_RECOVERING;
-  going_to_bootstrap = false;
 
+  finish_contexts(g_ceph_context, proposals, -EAGAIN);
   finish_contexts(g_ceph_context, waiting_for_commit, -EAGAIN);
   finish_contexts(g_ceph_context, waiting_for_active, -EAGAIN);
 }
@@ -1259,8 +1250,6 @@ void Paxos::propose_queued()
   assert(is_active());
   assert(!proposals.empty());
 
-  state = STATE_PREPARING;
-
   C_Proposal *proposal = static_cast<C_Proposal*>(proposals.front());
   assert(!proposal->proposed);
 
@@ -1273,6 +1262,7 @@ void Paxos::propose_queued()
   list_proposals(*_dout);
   *_dout << dendl;
 
+  state = STATE_UPDATING;
   begin(proposal->bl);
 }
 
