@@ -40,6 +40,8 @@ using namespace __gnu_cxx;
 #include "IndexManager.h"
 #include "ObjectMap.h"
 #include "SequencerPosition.h"
+#include "FDCache.h"
+#include "WBThrottle.h"
 
 #include "include/uuid.h"
 
@@ -198,6 +200,10 @@ private:
 
   friend ostream& operator<<(ostream& out, const OpSequencer& s);
 
+  Mutex fdcache_lock;
+  FDCache fdcache;
+  WBThrottle wbthrottle;
+
   Sequencer default_osr;
   deque<OpSequencer*> op_queue;
   uint64_t op_queue_len, op_queue_bytes;
@@ -250,37 +256,22 @@ private:
   void _journaled_ahead(OpSequencer *osr, Op *o, Context *ondisk);
   friend class C_JournaledAhead;
 
-  // flusher thread
-  Cond flusher_cond;
-  list<uint64_t> flusher_queue;
-  int flusher_queue_len;
-  void flusher_entry();
-  struct FlusherThread : public Thread {
-    FileStore *fs;
-    FlusherThread(FileStore *f) : fs(f) {}
-    void *entry() {
-      fs->flusher_entry();
-      return 0;
-    }
-  } flusher_thread;
-  bool queue_flusher(int fd, uint64_t off, uint64_t len);
-
   int open_journal();
 
-
   PerfCounters *logger;
 
 public:
   int lfn_find(coll_t cid, const hobject_t& oid, IndexedPath *path);
   int lfn_truncate(coll_t cid, const hobject_t& oid, off_t length);
   int lfn_stat(coll_t cid, const hobject_t& oid, struct stat *buf);
-  int lfn_open(coll_t cid, const hobject_t& oid, int flags, mode_t mode,
-	       IndexedPath *path);
-  int lfn_open(coll_t cid, const hobject_t& oid, int flags, mode_t mode,
-	       IndexedPath *path, Index *index);
-  int lfn_open(coll_t cid, const hobject_t& oid, int flags, mode_t mode);
-  int lfn_open(coll_t cid, const hobject_t& oid, int flags);
-  void lfn_close(int fd);
+  int lfn_open(
+    coll_t cid,
+    const hobject_t& oid,
+    bool create,
+    FDRef *outfd,
+    IndexedPath *path = 0,
+    Index *index = 0);
+  void lfn_close(FDRef fd);
   int lfn_link(coll_t c, coll_t cid, const hobject_t& o) ;
   int lfn_unlink(coll_t cid, const hobject_t& o, const SequencerPosition &spos);
 
@@ -314,10 +305,7 @@ public:
   }
   unsigned _do_transaction(Transaction& t, uint64_t op_seq, int trans_num);
 
-  int queue_transaction(Sequencer *osr, Transaction* t);
   int queue_transactions(Sequencer *osr, list<Transaction*>& tls,
-			 Context *onreadable, Context *ondisk=0,
-			 Context *onreadable_sync=0,
 			 TrackedOpRef op = TrackedOpRef());
 
   /**
@@ -367,12 +355,23 @@ public:
     return 0;
   }
   bool exists(coll_t cid, const hobject_t& oid);
-  int stat(coll_t cid, const hobject_t& oid, struct stat *st);
-  int read(coll_t cid, const hobject_t& oid, uint64_t offset, size_t len, bufferlist& bl);
+  int stat(
+    coll_t cid,
+    const hobject_t& oid,
+    struct stat *st,
+    bool allow_eio = false);
+  int read(
+    coll_t cid,
+    const hobject_t& oid,
+    uint64_t offset,
+    size_t len,
+    bufferlist& bl,
+    bool allow_eio = false);
   int fiemap(coll_t cid, const hobject_t& oid, uint64_t offset, size_t len, bufferlist& bl);
 
   int _touch(coll_t cid, const hobject_t& oid);
-  int _write(coll_t cid, const hobject_t& oid, uint64_t offset, size_t len, const bufferlist& bl);
+  int _write(coll_t cid, const hobject_t& oid, uint64_t offset, size_t len, const bufferlist& bl,
+      bool replica = false);
   int _zero(coll_t cid, const hobject_t& oid, uint64_t offset, size_t len);
   int _truncate(coll_t cid, const hobject_t& oid, uint64_t size);
   int _clone(coll_t cid, const hobject_t& oldoid, const hobject_t& newoid,
@@ -386,6 +385,7 @@ public:
 
   int _fgetattr(int fd, const char *name, bufferptr& bp);
   int _fgetattrs(int fd, map<string,bufferptr>& aset, bool user_only);
+  int _fsetattrs(int fd, map<string, bufferptr> &aset);
 
   void _start_sync();
 
@@ -403,6 +403,16 @@ public:
   }
   uuid_d get_fsid() { return fsid; }
 
+  // DEBUG read error injection, an object is removed from both on delete()
+  Mutex read_error_lock;
+  set<hobject_t> data_error_set; // read() will return -EIO
+  set<hobject_t> mdata_error_set; // getattr(),stat() will return -EIO
+  void inject_data_error(const hobject_t &oid);
+  void inject_mdata_error(const hobject_t &oid);
+  void debug_obj_on_delete(const hobject_t &oid);
+  bool debug_data_eio(const hobject_t &oid);
+  bool debug_mdata_eio(const hobject_t &oid);
+
   int snapshot(const string& name);
 
   // attrs
@@ -444,7 +454,11 @@ public:
   // omap (see ObjectStore.h for documentation)
   int omap_get(coll_t c, const hobject_t &hoid, bufferlist *header,
 	       map<string, bufferlist> *out);
-  int omap_get_header(coll_t c, const hobject_t &hoid, bufferlist *out);
+  int omap_get_header(
+    coll_t c,
+    const hobject_t &hoid,
+    bufferlist *out,
+    bool allow_eio = false);
   int omap_get_keys(coll_t c, const hobject_t &hoid, set<string> *keys);
   int omap_get_values(coll_t c, const hobject_t &hoid, const set<string> &keys,
 		      map<string, bufferlist> *out);
@@ -453,6 +467,7 @@ public:
   ObjectMap::ObjectMapIterator get_omap_iterator(coll_t c, const hobject_t &hoid);
 
   int _create_collection(coll_t c);
+  int _create_collection(coll_t c, const SequencerPosition &spos);
   int _destroy_collection(coll_t c);
   int _collection_add(coll_t c, coll_t ocid, const hobject_t& o,
 		      const SequencerPosition& spos);
@@ -471,10 +486,16 @@ private:
 		    const SequencerPosition &spos);
   int _omap_rmkeys(coll_t cid, const hobject_t &hoid, const set<string> &keys,
 		   const SequencerPosition &spos);
+  int _omap_rmkeyrange(coll_t cid, const hobject_t &hoid,
+		       const string& first, const string& last,
+		       const SequencerPosition &spos);
   int _omap_setheader(coll_t cid, const hobject_t &hoid, const bufferlist &bl,
 		      const SequencerPosition &spos);
   int _split_collection(coll_t cid, uint32_t bits, uint32_t rem, coll_t dest,
                         const SequencerPosition &spos);
+  int _split_collection_create(coll_t cid, uint32_t bits, uint32_t rem,
+			       coll_t dest,
+			       const SequencerPosition &spos);
 
   virtual const char** get_tracked_conf_keys() const;
   virtual void handle_conf_change(const struct md_config_t *conf,
@@ -483,20 +504,17 @@ private:
   bool m_filestore_btrfs_snap;
   float m_filestore_commit_timeout;
   bool m_filestore_fiemap;
-  bool m_filestore_flusher;
   bool m_filestore_fsync_flushes_journal_data;
   bool m_filestore_journal_parallel;
   bool m_filestore_journal_trailing;
   bool m_filestore_journal_writeahead;
   int m_filestore_fiemap_threshold;
-  bool m_filestore_sync_flush;
-  int m_filestore_flusher_max_fds;
-  int m_filestore_flush_min;
   double m_filestore_max_sync_interval;
   double m_filestore_min_sync_interval;
   bool m_filestore_fail_eio;
+  bool m_filestore_replica_fadvise;
   int do_update;
-  bool m_journal_dio, m_journal_aio;
+  bool m_journal_dio, m_journal_aio, m_journal_force_aio;
   std::string m_osd_rollback_to_cluster_snap;
   bool m_osd_use_stale_snap;
   int m_filestore_queue_max_ops;
