@@ -155,6 +155,7 @@ public:
       OP_SPLIT_COLLECTION = 35, // cid, bits, destination
       OP_SPLIT_COLLECTION2 = 36, /* cid, bits, destination
 				    doesn't create the destination */
+      OP_OMAP_RMKEYRANGE = 37,  // cid, oid, firstkey, lastkey
     };
 
   private:
@@ -357,6 +358,11 @@ public:
 	::decode(s, p);
 	return s;
       }
+      string get_key() {
+	string s;
+	::decode(s, p);
+	return s;
+      }
       void get_attrset(map<string,bufferptr>& aset) {
 	::decode(aset, p);
       }
@@ -606,6 +612,22 @@ public:
       ops++;
     }
 
+    /// Remove key range from hoid omap
+    void omap_rmkeyrange(
+      coll_t cid,             ///< [in] Collection containing hoid
+      const hobject_t &hoid,  ///< [in] Object from which to remove the omap
+      const string& first,    ///< [in] first key in range
+      const string& last      ///< [in] first key past range
+      ) {
+      __u32 op = OP_OMAP_RMKEYRANGE;
+      ::encode(op, tbl);
+      ::encode(cid, tbl);
+      ::encode(hoid, tbl);
+      ::encode(first, tbl);
+      ::encode(last, tbl);
+      ops++;
+    }
+
     /// Set omap header
     void omap_setheader(
       coll_t cid,             ///< [in] Collection containing hoid
