@@ -14,6 +14,9 @@
 #ifndef CEPH_REPLICATEDPG_H
 #define CEPH_REPLICATEDPG_H
 
+#include <boost/optional.hpp>
+
+#include "include/assert.h" 
 
 #include "PG.h"
 #include "OSD.h"
@@ -25,6 +28,18 @@
 #include "messages/MOSDSubOp.h"
 class MOSDSubOpReply;
 
+class ReplicatedPG;
+void intrusive_ptr_add_ref(ReplicatedPG *pg);
+void intrusive_ptr_release(ReplicatedPG *pg);
+uint64_t get_with_id(ReplicatedPG *pg);
+void put_with_id(ReplicatedPG *pg, uint64_t id);
+
+#ifdef PG_DEBUG_REFS
+  typedef TrackedIntPtr<ReplicatedPG> ReplicatedPGRef;
+#else
+  typedef boost::intrusive_ptr<ReplicatedPG> ReplicatedPGRef;
+#endif
+
 class PGLSFilter {
 protected:
   string xattr;
@@ -60,6 +75,7 @@ public:
 
 class ReplicatedPG : public PG {
   friend class OSD;
+  friend class Watch;
 public:  
 
   /*
@@ -95,24 +111,6 @@ public:
     
    */
 
-  struct SnapSetContext {
-    object_t oid;
-    int ref;
-    bool registered; 
-    SnapSet snapset;
-
-    SnapSetContext(const object_t& o) : oid(o), ref(0), registered(false) { }
-  };
-
-  struct ObjectState {
-    object_info_t oi;
-    bool exists;
-
-    ObjectState(const object_info_t &oi_, bool exists_)
-      : oi(oi_), exists(exists_) {}
-  };
-
-
   struct AccessMode {
     typedef enum {
       IDLE,
@@ -254,81 +252,6 @@ public:
     }
   };
 
-
-  /*
-   * keep tabs on object modifications that are in flight.
-   * we need to know the projected existence, size, snapset,
-   * etc., because we don't send writes down to disk until after
-   * replicas ack.
-   */
-  struct ObjectContext {
-    int ref;
-    bool registered; 
-    ObjectState obs;
-
-    SnapSetContext *ssc;  // may be null
-
-  private:
-    Mutex lock;
-  public:
-    Cond cond;
-    int unstable_writes, readers, writers_waiting, readers_waiting;
-
-    // set if writes for this object are blocked on another objects recovery
-    ObjectContext *blocked_by;      // object blocking our writes
-    set<ObjectContext*> blocking;   // objects whose writes we block
-
-    // any entity in obs.oi.watchers MUST be in either watchers or unconnected_watchers.
-    map<entity_name_t, OSD::Session *> watchers;
-    map<entity_name_t, Watch::C_WatchTimeout *> unconnected_watchers;
-    map<Watch::Notification *, bool> notifs;
-
-    ObjectContext(const object_info_t &oi_, bool exists_, SnapSetContext *ssc_)
-      : ref(0), registered(false), obs(oi_, exists_), ssc(ssc_),
-	lock("ReplicatedPG::ObjectContext::lock"),
-	unstable_writes(0), readers(0), writers_waiting(0), readers_waiting(0),
-	blocked_by(0) {}
-    
-    void get() { ++ref; }
-
-    // do simple synchronous mutual exclusion, for now.  now waitqueues or anything fancy.
-    void ondisk_write_lock() {
-      lock.Lock();
-      writers_waiting++;
-      while (readers_waiting || readers)
-	cond.Wait(lock);
-      writers_waiting--;
-      unstable_writes++;
-      lock.Unlock();
-    }
-    void ondisk_write_unlock() {
-      lock.Lock();
-      assert(unstable_writes > 0);
-      unstable_writes--;
-      if (!unstable_writes && readers_waiting)
-	cond.Signal();
-      lock.Unlock();
-    }
-    void ondisk_read_lock() {
-      lock.Lock();
-      readers_waiting++;
-      while (unstable_writes)
-	cond.Wait(lock);
-      readers_waiting--;
-      readers++;
-      lock.Unlock();
-    }
-    void ondisk_read_unlock() {
-      lock.Lock();
-      assert(readers > 0);
-      readers--;
-      if (!readers && writers_waiting)
-	cond.Signal();
-      lock.Unlock();
-    }
-  };
-
-
   /*
    * Capture all object state associated with an in-progress read or write.
    */
@@ -349,10 +272,17 @@ public:
     bool user_modify;     // user-visible modification
 
     // side effects
-    bool watch_connect, watch_disconnect;
-    watch_info_t watch_info;
+    list<watch_info_t> watch_connects;
+    list<watch_info_t> watch_disconnects;
     list<notify_info_t> notifies;
-    list<uint64_t> notify_acks;
+    struct NotifyAck {
+      boost::optional<uint64_t> watch_cookie;
+      uint64_t notify_id;
+      NotifyAck(uint64_t notify_id) : notify_id(notify_id) {}
+      NotifyAck(uint64_t notify_id, uint64_t cookie)
+	: watch_cookie(cookie), notify_id(notify_id) {}
+    };
+    list<NotifyAck> notify_acks;
     
     uint64_t bytes_written, bytes_read;
 
@@ -386,7 +316,6 @@ public:
       op(_op), reqid(_reqid), ops(_ops), obs(_obs), snapset(0),
       new_obs(_obs->oi, _obs->exists),
       modify(false), user_modify(false),
-      watch_connect(false), watch_disconnect(false),
       bytes_written(0), bytes_read(0),
       obc(0), clone_obc(0), snapset_obc(0), data_off(0), reply(NULL), pg(_pg) { 
       if (_ssc) {
@@ -516,15 +445,17 @@ protected:
   map<hobject_t, ObjectContext*> object_contexts;
   map<object_t, SnapSetContext*> snapset_contexts;
 
+  // debug order that client ops are applied
+  map<hobject_t, map<client_t, tid_t> > debug_op_order;
+
   void populate_obc_watchers(ObjectContext *obc);
-  void register_unconnected_watcher(void *obc,
-				    entity_name_t entity,
-				    utime_t expire);
-  void unregister_unconnected_watcher(void *obc,
-				      entity_name_t entity);
-  void handle_watch_timeout(void *obc,
-			    entity_name_t entity,
-			    utime_t expire);
+  void check_blacklisted_obc_watchers(ObjectContext *);
+  void check_blacklisted_watchers();
+  void get_watchers(list<obj_watch_item_t> &pg_watchers);
+  void get_obc_watchers(ObjectContext *obc, list<obj_watch_item_t> &pg_watchers);
+public:
+  void handle_watch_timeout(WatchRef watch);
+protected:
 
   ObjectContext *lookup_object_context(const hobject_t& soid) {
     if (object_contexts.count(soid)) {
@@ -750,7 +681,7 @@ protected:
 			 const hobject_t &last_backfill,
 			 interval_set<uint64_t>& data_subset,
 			 map<hobject_t, interval_set<uint64_t> >& clone_subsets);
-  void calc_clone_subsets(SnapSet& snapset, const hobject_t& poid, pg_missing_t& missing,
+  void calc_clone_subsets(SnapSet& snapset, const hobject_t& poid, const pg_missing_t& missing,
 			  const hobject_t &last_backfill,
 			  interval_set<uint64_t>& data_subset,
 			  map<hobject_t, interval_set<uint64_t> >& clone_subsets);
@@ -818,11 +749,6 @@ protected:
   void send_remove_op(const hobject_t& oid, eversion_t v, int peer);
 
 
-  void dump_watchers(ObjectContext *obc);
-  void remove_watcher(ObjectContext *obc, entity_name_t entity);
-  void remove_notify(ObjectContext *obc, Watch::Notification *notif);
-  void remove_watchers_and_notifies();
-
   struct RepModify {
     ReplicatedPG *pg;
     OpRequestRef op;
@@ -873,7 +799,7 @@ protected:
     }
   };
   struct C_OSD_AppliedRecoveredObject : public Context {
-    boost::intrusive_ptr<ReplicatedPG> pg;
+    ReplicatedPGRef pg;
     ObjectStore::Transaction *t;
     ObjectContext *obc;
     C_OSD_AppliedRecoveredObject(ReplicatedPG *p, ObjectStore::Transaction *tt, ObjectContext *o) :
@@ -883,7 +809,7 @@ protected:
     }
   };
   struct C_OSD_CommittedPushedObject : public Context {
-    boost::intrusive_ptr<ReplicatedPG> pg;
+    ReplicatedPGRef pg;
     OpRequestRef op;
     epoch_t epoch;
     eversion_t last_complete;
@@ -908,7 +834,7 @@ protected:
     }
   };
   struct C_OSD_CompletedPull : public Context {
-    boost::intrusive_ptr<ReplicatedPG> pg;
+    ReplicatedPGRef pg;
     hobject_t hoid;
     epoch_t epoch;
     C_OSD_CompletedPull(
@@ -925,7 +851,7 @@ protected:
   };
   friend class C_OSD_CompletedPull;
   struct C_OSD_AppliedRecoveredObjectReplica : public Context {
-    boost::intrusive_ptr<ReplicatedPG> pg;
+    ReplicatedPGRef pg;
     ObjectStore::Transaction *t;
     C_OSD_AppliedRecoveredObjectReplica(ReplicatedPG *p, ObjectStore::Transaction *tt) :
       pg(p), t(tt) {}
@@ -958,17 +884,6 @@ protected:
   virtual void _scrub_clear_state();
   virtual void _scrub_finish();
   object_stat_collection_t scrub_cstat;
-  virtual bool _report_snap_collection_errors(
-    const hobject_t &hoid,
-    int osd,
-    const map<string, bufferptr> &attrs,
-    const set<snapid_t> &snapcolls,
-    uint32_t nlinks,
-    ostream &out);
-  virtual void check_snap_collections(
-    ino_t hino, const hobject_t &hoid,
-    const map<string, bufferptr> &attrs,
-    set<snapid_t> *snapcolls);
 
   virtual void _split_into(pg_t child_pgid, PG *child, unsigned split_bits);
   void apply_and_flush_repops(bool requeue);
@@ -995,10 +910,7 @@ public:
   void do_sub_op_reply(OpRequestRef op);
   void do_scan(OpRequestRef op);
   void do_backfill(OpRequestRef op);
-  bool get_obs_to_trim(snapid_t &snap_to_trim,
-		       coll_t &col_to_trim,
-		       vector<hobject_t> &obs_to_trim);
-  RepGather *trim_object(const hobject_t &coid, const snapid_t &sn);
+  RepGather *trim_object(const hobject_t &coid);
   void snap_trimmer();
   int do_osd_ops(OpContext *ctx, vector<OSDOp>& ops);
 
@@ -1026,9 +938,7 @@ private:
   struct SnapTrimmer : public boost::statechart::state_machine< SnapTrimmer, NotTrimming > {
     ReplicatedPG *pg;
     set<RepGather *> repops;
-    vector<hobject_t> obs_to_trim;
     snapid_t snap_to_trim;
-    coll_t col_to_trim;
     bool need_share_pg_info;
     bool requeue;
     SnapTrimmer(ReplicatedPG *pg) : pg(pg), need_share_pg_info(false), requeue(false) {}
@@ -1037,23 +947,12 @@ private:
   } snap_trimmer_machine;
 
   /* SnapTrimmerStates */
-  struct RepColTrim : boost::statechart::state< RepColTrim, SnapTrimmer >, NamedState {
-    typedef boost::mpl::list <
-      boost::statechart::custom_reaction< SnapTrim >,
-      boost::statechart::transition< Reset, NotTrimming >
-      > reactions;
-    interval_set<snapid_t> to_trim;
-    RepColTrim(my_context ctx);
-    void exit();
-    boost::statechart::result react(const SnapTrim&);
-  };
-
   struct TrimmingObjects : boost::statechart::state< TrimmingObjects, SnapTrimmer >, NamedState {
     typedef boost::mpl::list <
       boost::statechart::custom_reaction< SnapTrim >,
       boost::statechart::transition< Reset, NotTrimming >
       > reactions;
-    vector<hobject_t>::iterator position;
+    hobject_t pos;
     TrimmingObjects(my_context ctx);
     void exit();
     boost::statechart::result react(const SnapTrim&);
@@ -1108,24 +1007,11 @@ public:
   void on_role_change();
   void on_change();
   void on_activate();
-  void on_removal();
+  void on_flushed();
+  void on_removal(ObjectStore::Transaction *t);
   void on_shutdown();
 };
 
-
-inline ostream& operator<<(ostream& out, ReplicatedPG::ObjectState& obs)
-{
-  out << obs.oi.soid;
-  if (!obs.exists)
-    out << "(dne)";
-  return out;
-}
-
-inline ostream& operator<<(ostream& out, ReplicatedPG::ObjectContext& obc)
-{
-  return out << "obc(" << obc.obs << ")";
-}
-
 inline ostream& operator<<(ostream& out, ReplicatedPG::RepGather& repop)
 {
   out << "repgather(" << &repop
