@@ -339,6 +339,20 @@ int RGWGetObj::verify_permission()
 }
 
 
+int RGWOp::verify_op_mask()
+{
+  uint32_t required_mask = op_mask();
+
+  ldout(s->cct, 20) << "required_mask= " << required_mask << " user.op_mask=" << s->user.op_mask << dendl;
+
+  if ((s->user.op_mask & required_mask) != required_mask) {
+    return -EPERM;
+  }
+
+  return 0;
+}
+
+
 int RGWGetObj::read_user_manifest_part(rgw_bucket& bucket, RGWObjEnt& ent, RGWAccessControlPolicy *bucket_policy, off_t start_ofs, off_t end_ofs)
 {
   ldout(s->cct, 0) << "user manifest obj=" << ent.name << dendl;
@@ -687,7 +701,7 @@ void RGWListBuckets::execute()
 
     done = (m.size() < read_count || (limit > 0 && total_count == limit));
 
-    if (m.size()) {
+    if (!m.empty()) {
       send_response_data(buckets);
 
       map<string, RGWBucketEnt>::reverse_iterator riter = m.rbegin();
@@ -1019,11 +1033,11 @@ class RGWPutObjProcessor_Aio : public RGWPutObjProcessor
   struct put_obj_aio_info pop_pending();
   int wait_pending_front();
   bool pending_has_completed();
+  int drain_pending();
 
 protected:
   uint64_t obj_len;
 
-  int drain_pending();
   int handle_obj_data(rgw_obj& obj, bufferlist& bl, off_t ofs, off_t abs_ofs, void **phandle);
   int throttle_data(void *handle);
 
@@ -1213,12 +1227,8 @@ int RGWPutObjProcessor_Atomic::do_complete(string& etag, map<string, bufferlist>
 
   store->set_atomic(s->obj_ctx, head_obj);
 
-  int r = drain_pending();
-  if (r < 0)
-    return r;
-
-  r = store->put_obj_meta(s->obj_ctx, head_obj, obj_len, NULL, attrs,
-                          RGW_OBJ_CATEGORY_MAIN, PUT_OBJ_CREATE, NULL, &first_chunk, &manifest, NULL, NULL);
+  int r = store->put_obj_meta(s->obj_ctx, head_obj, obj_len, NULL, attrs,
+                              RGW_OBJ_CATEGORY_MAIN, PUT_OBJ_CREATE, NULL, &first_chunk, &manifest, NULL, NULL);
 
   return r;
 }
@@ -1265,11 +1275,7 @@ int RGWPutObjProcessor_Multipart::do_complete(string& etag, map<string, bufferli
 {
   complete_parts();
 
-  int r = drain_pending();
-  if (r < 0)
-    return r;
-
-  r = store->put_obj_meta(s->obj_ctx, head_obj, s->obj_size, NULL, attrs, RGW_OBJ_CATEGORY_MAIN, 0, NULL, NULL, NULL, NULL, NULL);
+  int r = store->put_obj_meta(s->obj_ctx, head_obj, s->obj_size, NULL, attrs, RGW_OBJ_CATEGORY_MAIN, 0, NULL, NULL, NULL, NULL, NULL);
   if (r < 0)
     return r;
 
