@@ -14,18 +14,17 @@ public:
 };
 
 class RGWListBuckets_ObjStore_SWIFT : public RGWListBuckets_ObjStore {
-  int limit_max;
-  int limit;
-  string marker;
+  bool need_stats;
 public:
-  RGWListBuckets_ObjStore_SWIFT() {
-    limit_max = 10000;
-    limit = limit_max;
-  }
+  RGWListBuckets_ObjStore_SWIFT() : need_stats(true) {}
   ~RGWListBuckets_ObjStore_SWIFT() {}
 
   int get_params();
-  void send_response();
+  void send_response_begin(bool has_buckets);
+  void send_response_data(RGWUserBuckets& buckets);
+  void send_response_end();
+
+  bool should_get_stats() { return need_stats; }
 };
 
 class RGWListBucket_ObjStore_SWIFT : public RGWListBucket_ObjStore {
@@ -126,6 +125,14 @@ public:
   void send_response() {}
 };
 
+class RGWOptionsCORS_ObjStore_SWIFT : public RGWOptionsCORS_ObjStore {
+public:
+  RGWOptionsCORS_ObjStore_SWIFT() {}
+  ~RGWOptionsCORS_ObjStore_SWIFT() {}
+
+  void send_response();
+};
+
 class RGWHandler_ObjStore_SWIFT : public RGWHandler_ObjStore {
   friend class RGWRESTMgr_SWIFT;
 protected:
@@ -168,6 +175,7 @@ protected:
   RGWOp *op_put();
   RGWOp *op_delete();
   RGWOp *op_post();
+  RGWOp *op_options();
 public:
   RGWHandler_ObjStore_Bucket_SWIFT() {}
   virtual ~RGWHandler_ObjStore_Bucket_SWIFT() {}
@@ -186,6 +194,7 @@ protected:
   RGWOp *op_delete();
   RGWOp *op_post();
   RGWOp *op_copy();
+  RGWOp *op_options();
 public:
   RGWHandler_ObjStore_Obj_SWIFT() {}
   virtual ~RGWHandler_ObjStore_Obj_SWIFT() {}
